/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/ConfigurationManager.java,v $
*
* $Author: furykid $
* $Date: 2003/05/29 18:04:47 $
*
* modification history:
*
* $Log: ConfigurationManager.java,v $
* Revision 1.6  2003/05/29 18:04:47  furykid
* rebuild 290503
*
* Revision 1.5  2003/04/01 19:08:12  furykid
* cleanup
*
* Revision 1.4  2002/06/27 21:54:58  furykid
* adapted to new remotepackage requirements and the logging settings
*
* Revision 1.3  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import at.jps.threadpool.ThreadPool;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.NoSuchElementException;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Configuration File Reader Reads the mail.cfg file and fill the internal variables with the contens.
 * Other classes can use the getter functions to receive the values.
 * @author  Computer Mutter computer@mutter.com / Johannes Plachy
 * @version 0.99, 11/22/99
 */
final public class ConfigurationManager
{

    /**
     * Properties read from configuration file
     */
    private Properties               ivProp;

    /**
     * UserAlias list
     */
    private AliasList                ivAliasList;

    /**
     * Mail User list
     */
    private UserList                 ivUserList;

    /**
     * Sender for remote mail
     */
    private Sender                   ivSender;

    /**
     * handler for storing and retrieving mail
     */
    private MessageHandler           ivMessageHandler;

    /**
     * handler for storing and retrieving users
     */
    private UserHandler              ivUserHandler;

    /**
     * manager for all working threads
     */
    private ThreadPool               ivThreadPoolManager;

    /**
     * standard configuration file name
     */
    private static String            cvCFGFilename = MailServer.getAppName() + ".cfg";
    /**
     * retrieve default file name for cnfiguration information
     * @return filename
     */
    final public static String getDefaultCFGFilename()
    {
        return( "." + File.separator + MailServer.getAppName() + ".cfg" );
    }


    /**
     * initialize configuration from file
     * @param CFGFilename path to the configuration file
     */
    public ConfigurationManager( String CFGFilename ) throws Exception
    {
        cvCFGFilename = CFGFilename;

        load( CFGFilename );
    }


    /**
     * load initialization file
     * @param CFGFilename path to the configuration file
     */
    final private void load( String CFGFilename ) throws Exception
    {
        ivAliasList = new AliasList();
        ivProp = new Properties();

        FileInputStream   fileinputstream = null;

        try
        {
            fileinputstream = new FileInputStream( CFGFilename );

            ivProp.load( fileinputstream );
        }
        catch ( Exception x )
        {
            throw x;
        }
        finally
        {
            if ( fileinputstream != null )
            {
                fileinputstream.close();
            }
        }

        ivUserList = new UserList( getHome() );

        Vector   users = ivUserList.getVector();

        // new alias settings are username=alias1;alias2;etc

        for ( int i = 0; i < users.size(); i++ )
        {
            User     user = ( User )users.elementAt( i );
            String   alias = getProperty( "User." + user.getUsername(), null );

            if ( alias != null )
            {
                ivAliasList.add( user.getUsername(), alias );
            }
        }
    }


    /**
     * save configuration to file
     * @param CFGFilename path to the configuration file
     */
    final public void save() throws Exception
    {
        FileOutputStream  fileoutputstream = new FileOutputStream( cvCFGFilename );

        try
        {
            ivProp.store( fileoutputstream, MailServer.getAppName() + " - Java E-Mail Server (c) 1998 by Computer Mutter / (c) 1999 2000 by Johannes Plachy One Step Ahead Software" );
        }
        catch ( Exception x )
        {
            System.err.println( "Exception = " + x.toString() );
        }
        finally
        {
            fileoutputstream.close();
        }
    }


    /**
     * get list of registered mailusers
     * @return Userlist List of User
     */
    final public UserList getUserList()
    {
        return ivUserList;
    }


    /**
     * get Mailadministrator
     * @return name of administratoruser
     */
    final String getAdmin()
    {
        return getProperty( "User.admin", null );
    }


    /**
     * get list of Alias for Mailusers
     * @return list of Alias
     */
    final public AliasList getAliasList()
    {
        return ivAliasList;
    }


    /**
     * get List of POP3 servers that will be used to retrieve mail
     * @return list of serverdescriptions
     */
    final Vector getFetch()
    {
        Vector   vector = new Vector();
        int      i = 0;
        String   s;

        while ( ( s = getProperty( "Fetch" + i, null ) ) != null )
        {
            i++;

            vector.addElement( s );
        }

        return vector;
    }


    /**
     * get a POP3 server definition from configuration file that will be used to retrieve mail
     * @return server description
     */
    final String getFetch( int i )
    {
        return getProperty( "Fetch" + i, null );
    }


    /**
     * get path to working directory
     * @return path
     */
    final String getHome()
    {
        return getProperty( "Home", "." );
    }


    /**
     * get machinename
     * @return machinename
     */
    final String getLocalDomain()
    {
        return getProperty( "LocalDomain", "" );
    }


    /**
     * get DN Server
     * @return name of the DN server
     */
    final String getDNServer()
    {
        return getProperty( "DNServer", null );
    }


    /**
     * get Username of Default user, the one who gets mail for unknown users
     * @return name of registered default user
     */
    final String getUnknownUsername()
    {
        return getProperty( "User.unknown", null );
    }


    /**
     * get path to logfile that will be used to save Transaction logginginfo
     * @return path
     */
    final String getTransactionLogFile()
    {
        return getProperty( "Logfile.Transaction", "tx.log" );
    }


    /**
     * get path to logfile that will be used to save POP3 logginginfo
     * @return path
     */
    final String getPOP3LogFile()
    {
        return getProperty( "Logfile.POP3", "pop.log" );
    }


    /**
     * get path to logfile that will be used to save SMTP logginginfo
     * @return path
     */
    final String getSMTPLogFile()
    {
        return getProperty( "Logfile.SMTP", "smtp.log" );
    }


    /**
     * get path to logfile that will be used to save logginginfo for Sender Daemon
     * @return path
     */
    final String getSenderLogFile()
    {
        return getProperty( "Logfile.Sender", "sender.log" );
    }


    /**
     * get path to logfile that will be used to save logginginfo for Fetcher daemon
     * @return path
     */
    final String getFetcherLogFile()
    {
        return getProperty( "Logfile.Fetcher", "fetcher.log" );
    }


    /**
     * get path to logfile that will be used to save logginginfo for common messages
     * @return path
     */
    final String getStatusLogFile()
    {
        return getProperty( "Logfile.Status", "status.log" );
    }


    /**
     * returns the state of the logfile
     * @return enabled/disabled
     */

    final boolean isTransactionLogFileEnabled()
    {
        return getBooleanProperty( "Logfile.Transaction.enabled", true );
    }

    /**
     * returns the state of the logfile
     * @return enabled/disabled
     */

    final boolean isSMTPLogFileEnabled()
    {
        return getBooleanProperty( "Logfile.SMPTP.enabled", true );
    }

    /**
     * returns the state of the logfile
     * @return enabled/disabled
     */

    final boolean isPOP3LogFileEnabled()
    {
        return getBooleanProperty( "Logfile.POP3.enabled", true );
    }

    /**
     * returns the state of the logfile
     * @return enabled/disabled
     */

    final boolean isSenderLogFileEnabled()
    {
        return getBooleanProperty( "Logfile.Sender.enabled", true );
    }

    /**
     * returns the state of the logfile
     * @return enabled/disabled
     */

    final boolean isFetcherLogFileEnabled()
    {
        return getBooleanProperty( "Logfile.Fetcher.enabled", true );
    }

    /**
     * returns the state of the logfile
     * @return enabled/disabled
     */

    final boolean isStatusLogFileEnabled()
    {
        return getBooleanProperty( "Logfile.Status.enabled", true );
    }

    /**
     * set state of the logfile enabled or disabled
     *
     */
    final void setStatusLogFileState( boolean enable )
    {
        ivProp.put( "Logfile.Status.enabled", new Boolean( enable ).toString() );
    }

    /**
     * set state of the logfile enabled or disabled
     *
     */
    final void setFetcherLogFileState( boolean enable )
    {
        ivProp.put( "Logfile.Fetcher.enabled", new Boolean( enable ).toString() );
    }

    /**
     * set state of the logfile enabled or disabled
     *
     */
    final void setSenderLogFileState( boolean enable )
    {
        ivProp.put( "Logfile.Sender.enabled", new Boolean( enable ).toString() );
    }

    /**
     * set state of the logfile enabled or disabled
     *
     */
    final void setPOP3LogFileState( boolean enable )
    {
        ivProp.put( "Logfile.POP3.enabled", new Boolean( enable ).toString() );
    }

    /**
     * set state of the logfile enabled or disabled
     *
     */
    final void setSMTPLogFileState( boolean enable )
    {
        ivProp.put( "Logfile.SMPTP.enabled", new Boolean( enable ).toString() );
    }

    /**
     * set state of the logfile enabled or disabled
     *
     */
    final void setTransactionLogFileState( boolean enable )
    {
        ivProp.put( "Logfile.Transaction.enabled", new Boolean( enable ).toString() );
    }



    /**
     * get portnumber to be used for the local POP3 server
     * @return int portnumber
     */
    final int getLocalPop3ServerPort()
    {
        return getIntProperty( "POP3.localPort", Pop3Server.getDefaultPop3Port() );
    }


    /**
     * get portnumber to be used for the local SMTP server
     * @return int portnumber
     */
    final int getLocalSMTPServerSocket()
    {
        return getIntProperty( "SMTP.localPort", SmtpServer.getDefaultSmtpPort() );
    }


    /**
     * get portnumber to be used for the remote SMTP server ( the one that will get outbound software)
     * @return int portnumber
     */
    final int getRemoteSMTPServerSocket()
    {
        return getIntProperty( "SMTP.remotePort", SmtpServer.getDefaultSmtpPort() );
    }


    /**
     * retrieve state of RMI support. for remote administration with RMI
     * @return boolean true if
     */
    final boolean getEnableRMIInterface()
    {
        return getBooleanProperty( "RegisterRMIInterface", false );
    }


    /**
     * retrieve portnumber of remote POP3 servies
     * @param line taken from configuration file
     * @return int port number
     */
    final int getPop3Port( String s )
    {
        StringTokenizer   stringtokenizer = new StringTokenizer( s, ";" );

        stringtokenizer.nextToken();
        stringtokenizer.nextToken();
        stringtokenizer.nextToken();

        try
        {
            return Integer.parseInt( stringtokenizer.nextToken() );
        }
        catch ( Exception x )
        {

            // either not specified or no number ....

            return Pop3Server.getDefaultPop3Port();
        }
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     */
    final int getMaxWorkerThreads()
    {
        return getIntProperty( "THREADPOOL.max_threads", ThreadPool.MAX_THREADS );
    }


    /**
     * Method declaration
     *
     *
     * @param maxthreads
     *
     *
     */
    final void setMaxWorkerThreads( int maxthreads )
    {
        ivProp.put( "THREADPOOL.max_threads", Integer.toString( maxthreads ) );
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     */
    final int getSenderWaitDelay()
    {
        return getIntProperty( "Sender.wait_timeout", Event.getDefaultWaitTimeout() );
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     */
    final int getSenderReplyDelay()
    {
        return getIntProperty( "Sender.reply_timeout", Sender.getDefaultReceiveDelay() );
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     */
    final int getSMTPTimeout()
    {
        return getIntProperty( "SMTP.timeout", SmtpConnection.getDefaultTimeout() );
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     */
    final int getPOP3Timeout()
    {
        return getIntProperty( "POP3.timeout", Pop3Connection.getDefaultTimeout() );
    }


    /**
     * retrieve delay time between two fetch attempts from remote POP servers
     * @return int delay in minutes
     */
    final int getFetcherDelay()
    {
        return getIntProperty( "Fetcher.delay_time", Fetcher.getDefaultDelay() );
    }


    /**
     * returns information about the sender daemon
     * @return true if it should be enabled
     */
    final boolean getRunSenderDaemon()
    {
        return getBooleanProperty( "Sender.runDaemon", false );
    }


    /**
     * returns information about the fetcher daemon
     * @return true if fetcher should be enabled
     */
    final boolean getRunFetcherDaemon()
    {
        return getBooleanProperty( "Fetcher.runDaemon", false );
    }


    /**
     * retrieve password for given remote POP account
     * @param line from configuration file
     * @return String users password
     */
    final String getPop3Pass( String s )
    {
        String   result = null;

        try
        {
            StringTokenizer   stringtokenizer = new StringTokenizer( s, ";" );

            stringtokenizer.nextToken();
            stringtokenizer.nextToken();

            result = stringtokenizer.nextToken();
        }
        catch ( NoSuchElementException x )
        {
        }

        return result;
    }


    /**
     * retrieve name of a remote POP server
     * @param line from configuration file
     * @return String users password
     */
    final String getPop3Server( String s )
    {
        String   result = null;

        try
        {
            StringTokenizer   stringtokenizer = new StringTokenizer( s, ";" );

            result = stringtokenizer.nextToken();
        }
        catch ( NoSuchElementException x )
        {
        }

        return result;
    }


    /**
     * retrieve name of a remote POP server user
     * @param line from configuration file
     * @return String users name
     */
    final String getPop3User( String s )
    {
        String   result = null;

        try
        {
            StringTokenizer   stringtokenizer = new StringTokenizer( s, ";" );

            stringtokenizer.nextToken();

            result = stringtokenizer.nextToken();
        }
        catch ( NoSuchElementException x )
        {
        }

        return result;
    }


    /**
     * retrieve property value
     * @param property name
     * @return String value
     */
    final private String getProperty( String key, String defaultValue )
    {
        String   value = ivProp.getProperty( key );

        if ( value != null )
        {
            value.trim();
        }
        else
        {
            value = defaultValue;
        }

        return value;
    }


    /**
     * Method declaration
     * @param key
     * @param defaultValue
     * @return
     */
    final private int getIntProperty( String key, int defaultValue )
    {
        int   value = defaultValue;

        try
        {
            value = Integer.parseInt( getProperty( key, null ) );
        }
        catch ( Exception x )
        {
        }

        return value;
    }


    /**
     * Method declaration
     *
     *
     * @param key
     *
     * @param defaultValue
     *
     *
     * @return
     *
     */
    final private boolean getBooleanProperty( String key, boolean defaultValue )
    {
        boolean  value = defaultValue;

        try
        {
            value = Boolean.valueOf( getProperty( key, null ) ).booleanValue();
        }
        catch ( Exception x )
        {
        }

        return value;
    }


    /**
     * retrieve SMTP server for mail delivery
     * @return String Servername
     */
    final String getSMTPServer()
    {
        return getProperty( "SMTP.server", null );
    }


    /**
     * retrieve Sender daemon
     * @return Sender daemon or null
     */
    final Sender getSender()
    {
        return ivSender;
    }


    /**
     * Method declaration
     * @return
     */
    final boolean getShowStatusWindow()
    {
        return getBooleanProperty( "ShowStatusWindow", false );
    }


    /**
     * Method declaration
     * @param show
     */
    final void setShowStatusWindow( boolean show )
    {
        ivProp.put( "ShowStatusWindow", new Boolean( show ).toString() );
    }


    /**
     * retrieve administrator user
     * @return username
     */
    final void setAdmin( String s )
    {
        ivProp.put( "User.admin", s );
    }


    /**
     * store information about remote POP3 servers
     * @param int index
     * @param String Concatinated info (; delimited)
     */
    final void setFetch( int i, String s )
    {
        ivProp.put( "Fetch" + i, s );
    }


    /**
     * store information about Workingdirectory
     * @param String directory name
     */
    final void setHome( String s )
    {
        ivProp.put( "Home", s );
    }


    /**
     * store Computername
     * @param computer name
     */
    final void setLocalDomainName( String s )
    {
        ivProp.put( "LocalDomain", s );
    }


    /**
     * set DN Server
     * @return name of the DN server
     */
    final void setDNServer( String s )
    {
        ivProp.put( "DNServer", s );
    }


    /**
     * store SMTP server name
     * @param remote SMTP server name
     */
    final void setSMTPServer( String s )
    {
        ivProp.put( "SMTP.server", s );
    }


    /**
     * store default user
     * @param user name
     */
    final void setUnknownUsername( String s )
    {
        ivProp.put( "User.unknown", s );
    }


    /**
     * store information for Transaction Logfile
     * @param logfilename
     */
    final void setTransactionLogFile( String s )
    {
        ivProp.put( "Logfile.Transaction", s );
    }


    /**
     * store information for POP3 Logfile
     * @param logfilename
     */
    final void setPOP3LogFile( String s )
    {
        ivProp.put( "Logfile.POP3", s );
    }


    /**
     * store information for SMTP Logfile
     * @param logfilename
     */
    final void setSMTPLogFile( String s )
    {
        ivProp.put( "Logfile.SMTP", s );
    }


    /**
     * store information for Sender daemon Logfile
     * @param logfilename
     */
    final void setSenderLogFile( String s )
    {
        ivProp.put( "Logfile.Sender", s );
    }


    /**
     * store information for Fetcher daemon Logfile
     * @param logfilename
     */
    final void setFetcherLogFile( String s )
    {
        ivProp.put( "Logfile.Fetcher", s );
    }


    /**
     * store information for Status daemon Logfile
     * @param logfilename
     */
    final void setStatusLogFile( String s )
    {
        ivProp.put( "Logfile.Status", s );
    }


    /**
     * store information about local POP3 Server Port
     * @param Portnumber ( as String)
     */
    final void setLocalPop3ServerPort( String s )
    {
        ivProp.put( "POP3.localPort", s );
    }


    /**
     * store information about local SMTP Server Port
     * @param Portnumber ( as String)
     */
    final void setLocalSMTPServerSocket( String s )
    {
        ivProp.put( "SMTP.localPort", s );
    }


    /**
     * store information about remote SMTP Server Port
     * @param Portnumber ( as String)
     */
    final void setRemoteSMTPServerSocket( String s )
    {
        ivProp.put( "SMTP.remotePort", s );
    }


    /**
     * store information about RMI interface
     * @param true to enable RMI
     */
    final void setEnableRMIInterface( boolean enable )
    {
        ivProp.put( "RegisterRMIInterface", new Boolean( enable ).toString() );
    }


    /**
     * store information about Fetcher delay
     * @param int delay in minutes
     */
    final void setFetcherDelay( int delay )
    {
        ivProp.put( "Fetcher.delay_time", new Integer( delay ).toString() );
    }


    /**
     * store information about Sender daemon
     * @param true to enable daemon
     */
    final void setRunSenderDaemon( boolean enable )
    {
        ivProp.put( "Sender.runDaemon", new Boolean( enable ).toString() );
    }


    /**
     * store information about Fetcher daemon
     * @param true to enable daemon
     */
    final void setRunFetcherDaemon( boolean enable )
    {
        ivProp.put( "Fetcher.runDaemon", new Boolean( enable ).toString() );
    }


    /**
     * store information Sender ( runtime info)
     * @param new sender instance
     */
    final void setSender( Sender sender )
    {
        ivSender = sender;
    }


    /**
     * retrieve mailuser for which Email will be stored alias and unknown user are looked
     * @param user name to lookup
     */
    final String getReceiverName( String userOrAlias )
    {
        return getUserHandler().getReceiverName( userOrAlias);
    }


    /**
     * retrieve threadpool handler information ( runtime info)
     */
    final ThreadPool getThreadPool()
    {
        if ( ivThreadPoolManager == null )
        {
            ivThreadPoolManager = new ThreadPool( ThreadPool.MAX_THREADS );
        }

        return ivThreadPoolManager;
    }


    /**
     * retrieve handler information ( runtime info)
     */
    final MessageHandler getMessageHandler()
    {
        if ( ivMessageHandler == null )
        {
            ivMessageHandler = MessageHandlerFactory.getInstance().getMessageHandler( this );
        }

        return ivMessageHandler;
    }

    /**
     * retrieve handler information ( runtime info)
     */
    final UserHandler getUserHandler()
    {
        if ( ivUserHandler == null )
        {
            ivUserHandler = UserHandlerFactory.getInstance().getUserHandler( this );
        }

        return ivUserHandler;
    }


    /**
     * return settingsinfo as string
     * @return settings as String
     */
    final public String toString()
    {
        return "Host = " + getLocalDomain() + "\nSmtp = " + getSMTPServer() + "\nHome = " + getHome() + "\nAdmin = " + getAdmin();
    }


    /**
     * retrieve information about JDBC enabling
     * @return true if JDBC is tu be used
     */
    public boolean isJDBCendabled()
    {
        return getBooleanProperty( "JDBC.enabled", false );
    }


    /**
     * set JDBC enabled flag
     * @param true to enable JDBC
     */
    public void setJDBCendabled( boolean enable )
    {
        ivProp.put( "JDBC.enabled", new Boolean( enable ).toString() );
    }


    /**
     * set JDBC driver class name
     * @param String full qualified classname of JDBC driver
     */
    public void setJDBCDriver( String newJDBCDriver )
    {
        ivProp.put( "JDBC.driver", newJDBCDriver );
    }


    /**
     * get JDBC driver class name
     * @return String full qualified classname of JDBC driver
     */
    public String getJDBCDriver()
    {
        return getProperty( "JDBC.driver", null );
    }


    /**
     * set JDBC URL string
     * @param String URL to JDBC server
     */
    public void setJDBCURL( String newJDBCServerURL )
    {
        ivProp.put( "JDBC.URL", newJDBCServerURL );
    }


    /**
     * get SMTPFilter class whichc checks for Mailserver misuse in SMTP connections
     * @return
     */
    public String getSMTPFilterClass()
    {
        return getProperty( "SMTP.filterClass", null );
    }


    /**
     * set new Class that checks if the mailserver is getting misused in SMTP connections
     * @param newSMTPFilterClass
     */
    public void setSMTPFilterClass( String newSMTPFilterClass )
    {
        ivProp.put( "SMTP.filterClass", newSMTPFilterClass );
    }


    /**
     * get JDBC URL string
     * @return String URL to JDBC server
     */
    public String getJDBCURL()
    {
        return getProperty( "JDBC.URL", null );
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     */
    public boolean getBigBrother()
    {
        return getBooleanProperty( "BigBrother", false );
    }


    /**
     * provide an interface for checking if the sender daemon is running
     * @return boolean true if daemon is running
     */
    public boolean isSenderRunning()
    {
        return( getRunSenderDaemon() && getSender() != null );
    }


    /**
     * provide an interface for checking if the fetcher daemon is running
     * @return boolean true if daemon is running
     */
    public boolean isFetcherRunning()
    {
        Vector   v = getFetch();

        return( getRunFetcherDaemon() && ( v != null ) && ( v.size() > 0 ) );
    }


    /**
     * Method declaration
     * @return
     */
    public OutgoingMessagesManager getMessagesManager()
    {
        return OutgoingMessagesManager.getInstance( this );
    }


    /**
     * set settings
     * @param settings class
     * @param true if settings should be stored to disk
     */
    public void setSettings( Settings settings, boolean savePersistent )
    {
        try
        {
            setHome( settings.getHomeDirectory() );
            setLocalDomainName( settings.getLocalDomainName() );
            setSMTPServer( settings.getSMTPServer() );
            setUnknownUsername( settings.getUnknownUsername() );
            setPOP3LogFile( settings.getPOP3LogFile() );
            setSMTPLogFile( settings.getSMTPLogFile() );
            setSenderLogFile( settings.getSenderLogFile() );
            setFetcherLogFile( settings.getFetcherLogFile() );
            setStatusLogFile( settings.getStatusLogFile() );
            setTransactionLogFile( settings.getTransactionLogFile() );
            setLocalPop3ServerPort( Integer.toString( settings.getLocalPop3Port() ) );
            setLocalSMTPServerSocket( Integer.toString( settings.getLocalSMTPPort() ) );
            setRemoteSMTPServerSocket( Integer.toString( settings.getRemoteSMTPPort() ) );
            setRunSenderDaemon( settings.isRunSenderDaemon() );
            setRunFetcherDaemon( settings.isRunFetcherDaemon() );
            setJDBCendabled( settings.isUseJDBC() );
            setJDBCDriver( settings.getJDBCDriver() );
            setJDBCURL( settings.getJDBCURL() );
            setAdmin( settings.getAdmin() );
            setDNServer( settings.getDNServer() );
            setFetcherDelay( settings.getFetcherDelay() );
            setMaxWorkerThreads( settings.getMaxThreads() );
            setShowStatusWindow( settings.isShowStatusWindow());


            // save userinfo
            //

            Vector   users = new Vector();

            for ( int idx = 0; idx < settings.getUserList().size(); idx++ )
            {
                UserAndAlias   uaa = ( UserAndAlias )settings.getUserList().elementAt( idx );

                users.addElement( new User( uaa.getUserName().toLowerCase(), uaa.getPassword().toLowerCase() ) );

                // and alias

                if ((uaa != null) && ( uaa.getAliasList() != null))
                {
                    ivProp.put( "User." + uaa.getUserName(), uaa.getAliasList() );
                }
            }

            ivUserList.setVector( users );

            // save remote POP serversettings

            Vector   popSettings = settings.getRemotePopAccounts();

            for ( int idx = 0; idx < popSettings.size(); idx++ )
            {
                setFetch( idx, ( String )popSettings.elementAt( idx ) );
            }

            if ( savePersistent )
            {
                ivUserList.save();
                save();
            }
        }
        catch ( Exception x )
        {
            System.err.println( x );

            if (Debug.enabled == true)
            {
                x.printStackTrace();
            }

        }
    }


    /**
     * get settings
     * @return settings
     */
    public Settings getSettings()
    {
        Settings settings = new Settings();

        settings.setHomeDirectory( getHome() );
        settings.setLocalDomainName( getLocalDomain() );
        settings.setSMTPServer( getSMTPServer() );
        settings.setUnknownUsername( getUnknownUsername() );
        settings.setPOP3LogFile( getPOP3LogFile() );
        settings.setSMTPLogFile( getSMTPLogFile() );
        settings.setSenderLogFile( getSenderLogFile() );
        settings.setFetcherLogFile( getFetcherLogFile() );
        settings.setStatusLogFile( getStatusLogFile() );
        settings.setTransactionLogFile( getTransactionLogFile() );
        settings.setLocalPop3Port( getLocalPop3ServerPort() );
        settings.setLocalSMTPPort( getLocalSMTPServerSocket() );
        settings.setRemoteSMTPPort( getRemoteSMTPServerSocket() );
        settings.setRunSenderDaemon( getRunSenderDaemon() );
        settings.setRunFetcherDaemon( getRunFetcherDaemon() );
        settings.setUseJDBC( isJDBCendabled() );
        settings.setJDBCDriver( getJDBCDriver() );
        settings.setJDBCURL( getJDBCURL() );
        settings.setDNServer( getDNServer() );
        settings.setFetcherDelay( getFetcherDelay() );
        settings.setAdmin( getAdmin() );
        settings.setMaxThreads( getMaxWorkerThreads() );
        settings.setShowStatusWindow( getShowStatusWindow() );

        Vector   userinfo = new Vector( ivUserList.size() );

        for ( int i = 0; i < ivUserList.size(); i++ )
        {
            UserAndAlias   uaa = new UserAndAlias();

            uaa.setUserName( ivUserList.getUser( i ) );
            uaa.setPassword( ivUserList.getPassword( ivUserList.getUser( i ) ) );
            uaa.setAliasList( getProperty( "User." + uaa.getUserName(), null ) );
            userinfo.addElement( uaa );
        }

        settings.setUserList( userinfo );

        // build remote POP list

        settings.setRemotePopAccounts( getFetch() );

        // get statistics

        if ( MailStatistic.isInitialized() )
        {
            settings.setMailsIn( MailStatistic.getInstance().getNrOfIncomingMail() );
            settings.setMailsOut( MailStatistic.getInstance().getNrOfOutgoingMail() );
            settings.setSizeofMailsIn( MailStatistic.getInstance().getSizeOfIncomingMail() );
            settings.setSizeofMailsOut( MailStatistic.getInstance().getSizeOfOutgoingMail() );
            settings.setUpSince( MailStatistic.getInstance().getStartTime() );
        }

        return settings;
    }


    /**
     * Method declaration
     * @param args
     */
    final static void main( String args[] )
    {
        try
        {
            ConfigurationManager configurationManager = new ConfigurationManager( ConfigurationManager.getDefaultCFGFilename() );

            configurationManager.save();
        }
        catch ( Exception exception )
        {
            System.out.println( exception );
        }
    }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

