/*--- formatted by Jindent 2.1, (www.c-lab.de/~jindent) ---*/

/*
 * Copyright (c) 1999 / 2000  Johannes Plachy & Computer Mutter
 *
 * JMailSrv@jps.at
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * If you modify this file, please send us a copy.
 */

/********************************************************************************************
* CVS Info
*
* $Source: /cvsroot/jmailsrv/jmailserver/at/jps/mailserver/MailServer.java,v $
*
* $Author: furykid $
* $Date: 2003/04/01 20:35:58 $
*
* modification history:
*
* $Log: MailServer.java,v $
* Revision 1.3  2003/04/01 20:35:58  furykid
* cleanup
*
* Revision 1.2  2002/01/20 16:07:13  furykid
* added new header for cvs information
*

*********************************************************************************************/


package at.jps.mailserver;

import at.jps.mailserver.gui.StatusFrame;

/**
 * Mailserver is the core class for running all services at once if enabled in the configuration file
 * the Sender and Fetcher daemons are started which send and receive Mail from and to the internet
 * The local POP3 and SMTP services are started also (c) 1998 Computer Mutter computer@mutter.com; 1999 Johannes Plachy
 */
final public class MailServer
{

   /**
    * application name
    */
   private final static String   cvAppName = "JMailSrv";

   /**
    * configuration data
    */
   private ConfigurationManager  configurationManager;

   /**
    * worker thread for POP3
    */
   private Pop3Server            popserver;

   /**
    * worker thread for SMTP
    */
   private SmtpServer            smtpserver;

   /**
    * worker thread for sending mail to remote addresses
    */
   private Sender                sender;

   /**
    * worker thread for receiving mail from remote accounts
    */
   private Fetcher               fetcher;

   /**
    * instance of this singleton class
    */
   private static MailServer     cvMailServer = null;

   /**
    * returns the application name
    * @return application name
    */
   final public static String getAppName()
   {
      return cvAppName;
   }


   /**
    * create instance of the mailserver and initializes the instance with configuration data
    * @param configuration
    */
   private MailServer( String CFGFilename )
   {
      cvMailServer = this;

      initialize( CFGFilename );
   }


   /**
    * returns instance of this class
    * @return instance
    */
   public static MailServer getInstance()
   {
      return cvMailServer;
   }


   /**
    * returns configuration data
    * @return configuration
    */
   public ConfigurationManager getConfiguration()
   {
      return configurationManager;
   }


   /**
    * this function supports the runtime modification of configuration data if a new configuration information is passed all
    * worker threads are notified and will use the new settings if possible
    * @param new configuration data
    */
   final public void setNewCFGFile( String CFGFilename )
   {
      try
      {
         configurationManager = new ConfigurationManager( CFGFilename );

         if ( configurationManager == null )
         {
            return;
         }

         // update all dependent modules

         ErrorReporter.getInstance().updateCFG( configurationManager );

         if ( popserver != null )
         {
            popserver.updateCFG( configurationManager );
         }

         if ( smtpserver != null )
         {
            smtpserver.updateCFG( configurationManager );
         }

         if ( sender != null )
         {
            sender.updateCFG( configurationManager );
         }
         else     // maybe start now ?
         if ( configurationManager.getRunSenderDaemon() )
         {
            sender = Sender.startSender( configurationManager );
         }

         if ( fetcher != null )
         {
            fetcher.updateCFG( configurationManager );
         }
         else     // maybe start now ?
         if ( configurationManager.getRunFetcherDaemon() )
         {
            fetcher = Fetcher.startFetcher( configurationManager );
         }
      }
      catch ( Exception exception )
      {
         ErrorReporter.getInstance().writeStatusMsg( exception.toString(), "Error" );
      }
   }


   /**
    * helper function that wraps the creation of a mailserver
    * @param new configuration data
    */
   public static MailServer startServer( String CFGFilename )
   {
      return new MailServer( ( ( CFGFilename != null ) && ( CFGFilename.length() > 0 ) ) ? CFGFilename : ConfigurationManager.getDefaultCFGFilename() );
   }


   /**
    * create all worker threads and error logging facility
    * @param configuration data
    */
   final private void initialize( String CFGFilename )
   {
      try
      {
         configurationManager = new ConfigurationManager( CFGFilename );

         ErrorReporter.getInstance().updateCFG( configurationManager );

         ErrorReporter.getInstance().writeStatusMsg( "Configuration read" );

         AutoresponderController.getInstance().updateCFG( configurationManager );

         ErrorReporter.getInstance().writeStatusMsg( "Autoresponsecontroller initialized" );

         popserver = Pop3Server.startServer( configurationManager );
         smtpserver = SmtpServer.startServer( configurationManager );

         ErrorReporter.getInstance().writeStatusMsg( "POP and SMTP daemons started !" );

         if ( configurationManager.getRunSenderDaemon() )
         {
            sender = Sender.startSender( configurationManager );

            ErrorReporter.getInstance().writeStatusMsg( "Sender daemon started !" );
         }

         if ( configurationManager.getRunFetcherDaemon() )
         {
            fetcher = Fetcher.startFetcher( configurationManager );

            ErrorReporter.getInstance().writeStatusMsg( "Fetcher daemon started !" );
         }

         if ( configurationManager.getEnableRMIInterface() )
         {
            MailServerAdminImpl.registerServerAdmin( "localhost", configurationManager.getHome() );
            ErrorReporter.getInstance().writeStatusMsg( ">> RMI Interface Registered successfully <<" );
         }

         if ( configurationManager.getShowStatusWindow() )
         {
            StatusFrame.showStatusWindow( true );
         }

      }
      catch ( Exception exception )
      {
         if ( configurationManager == null )
         {
            System.err.println( "Configuration file missing or invalid" );
         }
         else
         {
            ErrorReporter.getInstance().writeStatusMsg( exception.toString(), "Error" );
         }
      }

      ErrorReporter.getInstance().writeStatusMsg( "---- initialization complete ! ----" );
   }


   /**
    * Method declaration
    * @param args
    */
   public static void main( String args[] )
   {
      System.out.println( Version.getVersion( "Mailserver" ) );
      startServer( ( ( args.length >= 1 ) ? args[ 0 ] : ConfigurationManager.getDefaultCFGFilename() ) );
   }


}



/*--- formatting done in "JPS Convention" style on 08-20-2001 ---*/

